//-----------------------------------------------------------------------------
// File: AMT.cs
//
// AMT main form
//
// 
//-----------------------------------------------------------------------------


//#define DEBUG_VS   // Uncomment this line to debug vertex shaders 
//#define DEBUG_PS   // Uncomment this line to debug pixel shaders 

using System;
using System.Windows.Forms;
using System.Drawing;
using SlimDX;
using SlimDX.Direct3D9;
using Font = SlimDX.Direct3D9.Font;
using Microsoft.Samples.DirectX.UtilityToolkit;
using System.Diagnostics;
using Allegiance;
using Mindshifter;

namespace MDLEditor
{
    /// <summary>EnhancedMesh Sample Class</summary>
    public class AMTForm : Form, IFrameworkCallback, IDeviceCreation
    {
        #region Creation
        /// <summary>maint AMT window</summary>
        public AMTForm(Framework f) 
        { 
            // Store framework
            sampleFramework = f;
            Configuration.EnableObjectTracking = true;
            InitializeComponent();

            // todo fetch from config
            //ofdMain.InitialDirectory = @"C:\dev\alleg\AllegianceSource1p2\src\Artwork\256";
            ofdMain.InitialDirectory = @"C:\Program Files\Microsoft Games\Allegiance\artwork";
            ofdCVH.InitialDirectory = @"C:\Program Files\Microsoft Games\Allegiance\artwork";

            tbSize = new TrackBar();
            tbSize.AutoSize = false; // the TrackBar wants to be 45px high by default, turning AutoSize = false fixes this
            tbSize.Height = 16;
            tbSize.Minimum = 1;
            tbSize.Maximum = 20;
            toolStrip1.Items.Add(new ToolStripControlHost(tbSize));


            this.MouseWheel += new MouseEventHandler(panelRenderTarget_MouseWheel);
            ToolStripItem tsi;
            tsi = cmsNode.Items.Add("Solid"); tsi.Tag = RenderStyle.Solid;
            tsi = cmsNode.Items.Add("Wireframe"); tsi.Tag = RenderStyle.Wireframe;
            tsi = cmsNode.Items.Add("Hide"); tsi.Tag = RenderStyle.Hide;
            tsi = null;

            // device indep object
            lightmat = new Material();
            lightmat.Diffuse = new Color4(1.0f, 1.0f, 1.0f, 1.0f);
            lightmat.Power = 1.0f;

            cvhmat = new Material();
            cvhmat.Diffuse = new Color4(0.6f, 1.0f, 1.0f, 0.0f);

            manipulator = new Manipulator(sampleFramework, camera);
            manipulator.EnabledModes = TransformationMode.TranslationAxis;
            manipulator.Settings.DrawBoundsEnabled = true;

            camera.SetButtonMasks((int)MouseButtonMask.Right, (int)MouseButtonMask.Wheel, (int)MouseButtonMask.None);

            tsbManipTransAxis.Tag = TransformationMode.TranslationAxis;
            tsbManipTransAxis.Checked = true;
            tsbManipTransPlane.Tag = TransformationMode.TranslationPlane;
            tsbManipRotate.Tag = TransformationMode.Rotation;
            tsbManipScaleAxis.Tag = TransformationMode.ScaleAxis;
            tsbManipScalePlane.Tag = TransformationMode.ScalePlane;
            tsbManipScaleUni.Tag = TransformationMode.ScaleUniform;
        }

        #endregion
        // Variables
        private Framework sampleFramework = null; // Framework for samples
        private Font statsFont = null; // Font for drawing text
        private Sprite textSprite = null; // Sprite for batching text calls
        //private Effect effect = null; // D3DX Effect Interface
        private MVICamera camera = new MVICamera(); // A model viewing camera exposing ICamera
        private bool isHelpShowing = true; // If true, renders the UI help text
        private Manipulator manipulator;

        // Sample specific variables
        private AnimationFrame rootFrame;
        private AnimationFrame selectedFrame;

        public AnimationFrame SelectedFrame
        {
            get { return selectedFrame; }
            set { 
                selectedFrame = value;
                if (manipulator != null)
                    manipulator.Transform = selectedFrame;
            }
        }
        private AnimationController controller;
        private bool isDestroyed = false;
        private CVH cvh;

        private Mesh arrowmesh;
        private Mesh spheremesh;
        private Mesh garagemesh;
        private Mesh planemesh;
        private Material lightmat;
        private Material cvhmat;

        private Vector3 objectCenter; // Center of bounding sphere of object
        private float objectRadius; // Radius of bounding sphere of object
        public Texture defaultTexture = null; // Default white texture
        private bool isWireframe = false;
        private bool bShowLights = true;

        private TrackBar tbSize;
        private ToolStrip toolStrip1;
        private ToolStripButton tsbWireframe;
        private ToolStripButton newToolStripButton;
        private ToolStripButton openToolStripButton;
        private ToolStripButton saveToolStripButton;
        private ToolStripSeparator toolStripSeparator;
        private ToolStripButton cutToolStripButton;
        private ToolStripButton copyToolStripButton;
        private ToolStripButton pasteToolStripButton;
        private ToolStripSeparator toolStripSeparator1;
        private ToolStripButton helpToolStripButton;
        private ToolStripSeparator toolStripSeparator2;
        private ContextMenuStrip cmsNode;
        private System.ComponentModel.IContainer components;
        private ListView listView1;
        private SplitContainer splitContainer3;
        private ImageList iconList;
        private ToolStripSeparator toolStripSeparator3;
        private ToolStripButton toolStripButton2;
        private SplitContainer splitContainer1;
        private Panel panelRenderTarget;
        private OpenFileDialog ofdMain;
        private ToolStripButton tsbCenterView;
        private ToolStripSeparator toolStripSeparator4;
        private OpenFileDialog ofdCVH;
        private ToolStripButton tsbShowLights;
        private ToolStripSplitButton tsbAdd;
        private ToolStripMenuItem toolStripMenuItem1;
        private ToolStripMenuItem lightToolStripMenuItem;
        private ToolStripMenuItem effectToolStripMenuItem;
        private ToolStripMenuItem launchToolStripMenuItem;
        private ToolStripMenuItem garageToolStripMenuItem;
        private ToolStripMenuItem mountToolStripMenuItem;
        private ToolStripMenuItem toolStripMenuItem2;
        private ToolStripMenuItem importToolStripMenuItem;
        private ToolStripMenuItem computeToolStripMenuItem;
        private ToolStripSeparator toolStripSeparator5;
        private ToolStripButton tsbManipTransAxis;
        private ToolStripButton tsbManipRotate;
        private ToolStripButton tsbManipTransPlane;
        private ToolStripButton tsbManipScaleAxis;
        private ToolStripButton tsbManipScalePlane;
        private ToolStripSeparator toolStripSeparator6;
        private ToolStripButton tsbManipScaleUni;
        private TreeView tvFrames;

        /// <summary>
        /// Called during device initialization, this code checks the device for some 
        /// minimum set of capabilities, and rejects those that don't pass by returning false.
        /// </summary>
        public bool IsDeviceAcceptable(Capabilities caps, Format adapterFormat, Format backBufferFormat, bool windowed)
        {
            // No fallback defined by this app, so reject any device that 
            // doesn't support at least ps2.0
            if (caps.PixelShaderVersion < new Version(2,0))
                return false;

            // Skip back buffer formats that don't support alpha blending
            if (!Framework.Direct3D9Object.CheckDeviceFormat(caps.AdapterOrdinal, caps.DeviceType, adapterFormat, 
                Usage.QueryPostPixelShaderBlending, ResourceType.Texture, backBufferFormat))
                return false;

            return true;
        }

        /// <summary>
        /// This callback function is called immediately before a device is created to allow the 
        /// application to modify the device settings. The supplied settings parameter 
        /// contains the settings that the framework has selected for the new device, and the 
        /// application can make any desired changes directly to this structure.  Note however that 
        /// the sample framework will not correct invalid device settings so care must be taken 
        /// to return valid device settings, otherwise creating the Device will fail.  
        /// </summary>
        public void ModifyDeviceSettings(DeviceSettings settings, Capabilities caps)
        {
            // If device doesn't support HW T&L or doesn't support 1.1 vertex shaders in HW 
            // then switch to SWVP.
            if (!((caps.DeviceCaps & DeviceCaps.HWTransformAndLight) !=0) ||
                (caps.VertexShaderVersion < new Version(1,1)) )
            {
                settings.BehaviorFlags = CreateFlags.SoftwareVertexProcessing;
            }
            else
            {
                settings.BehaviorFlags = CreateFlags.HardwareVertexProcessing;
            }

            // This application is designed to work on a pure device by not using 
            // any get methods, so create a pure device if supported and using HWVP.
            if ( ((caps.DeviceCaps & DeviceCaps.PureDevice)!=0) && 
                ((settings.BehaviorFlags & CreateFlags.HardwareVertexProcessing) != 0 ) )
                settings.BehaviorFlags |= CreateFlags.PureDevice;

            // Debugging vertex shaders requires either REF or software vertex processing 
            // and debugging pixel shaders requires REF.  
#if(DEBUG_VS)
            if (settings.DeviceType != DeviceType.Reference )
            {
                settings.BehaviorFlags &= ~CreateFlags.HardwareVertexProcessing;
                settings.BehaviorFlags |= CreateFlags.SoftwareVertexProcessing;
            }
#endif
#if(DEBUG_PS)
            settings.DeviceType = DeviceType.Reference;
#endif

            // For the first device created if its a REF device, optionally display a warning dialog box
            if (settings.DeviceType == DeviceType.Reference)
            {
                Utility.DisplaySwitchingToRefWarning(sampleFramework, "AMT II");
            }
        }

        /// <summary>
        /// This event will be fired immediately after the Direct3D device has been 
        /// created, which will happen during application initialization and windowed/full screen 
        /// toggles. This is the best location to create Pool.Managed resources since these 
        /// resources need to be reloaded whenever the device is destroyed. Resources created  
        /// here should be released in the Disposing event. 
        /// </summary>
        private void OnCreateDevice(object sender, DeviceEventArgs e)
        {
            // Initialize the stats font
            statsFont = MDLResourceCache.GetGlobalInstance().CreateFont(e.Device, 15, 0, FontWeight.Bold, 1, false, CharacterSet.Default,
                Precision.Default, FontQuality.Default, PitchAndFamily.DontCare | PitchAndFamily.Default
                , "Arial");

            // Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders with the 
            // shader debugger. Debugging vertex shaders requires either REF or software vertex 
            // processing, and debugging pixel shaders requires REF.  The 
            // ShaderFlags.Force*SoftwareNoOptimizations flag improves the debug experience in the 
            // shader debugger.  It enables source level debugging, prevents instruction 
            // reordering, prevents dead code elimination, and forces the compiler to compile 
            // against the next higher available software target, which ensures that the 
            // unoptimized shaders do not exceed the shader model limitations.  Setting these 
            // flags will cause slower rendering since the shaders will be unoptimized and 
            // forced into software.  See the DirectX documentation for more information about 
            // using the shader debugger.

            // MDX to SlimDX port - removed
            //ShaderFlags shaderFlags = ShaderFlags.NotCloneable;
            // End of MDX to SlimDX port
            //ShaderFlags shaderFlags = ShaderFlags.None;
#if(DEBUG_VS)
            shaderFlags |= ShaderFlags.ForceVertexShaderSoftwareNoOptimizations;
#endif
#if(DEBUG_PS)
            shaderFlags |= ShaderFlags.ForcePixelShaderSoftwareNoOptimizations;
#endif
            // Read the D3DX effect file
            //string path = Utility.FindMediaFile("EnhancedMesh.fx");
            //effect = ResourceCache.GetGlobalInstance().CreateEffectFromFile(e.Device,
            //    path, null, null, shaderFlags, null);

            // Create the 1x1 white default texture
            defaultTexture = new Texture(e.Device, 1, 1, 1, Usage.None, Format.A8R8G8B8, Pool.Managed);
            /*using*/ DataRectangle data = defaultTexture.LockRectangle(0, LockFlags.None);
            {
                // Write a single white pixel
                data.Data.Write(0xffffffff);
                // Now unlock
                defaultTexture.UnlockRectangle(0);
            }

            string currentFolder = System.IO.Directory.GetCurrentDirectory();
            string path = Utility.FindMediaFile("UI\\arrow.x");
            System.IO.FileInfo info = new System.IO.FileInfo(path);
            System.IO.Directory.SetCurrentDirectory(info.Directory.FullName);
            arrowmesh = Mesh.FromFile(e.Device, path, MeshFlags.Managed);

            path = Utility.FindMediaFile("UI\\plane.x");
            info = new System.IO.FileInfo(path);
            System.IO.Directory.SetCurrentDirectory(info.Directory.FullName);
            planemesh = Mesh.FromFile(e.Device, path, MeshFlags.Managed);

            spheremesh = Mesh.CreateSphere(e.Device, 1.0f, 12, 6);

            garagemesh = new Mesh(e.Device, 6, 5, MeshFlags.SystemMemory, VertexFormat.Position);
            DataStream ds = garagemesh.LockVertexBuffer(LockFlags.None);
            ds.Write<Vector3>(new Vector3(0, 0, 1));
            ds.Write<Vector3>(new Vector3(-1, 1, 0));
            ds.Write<Vector3>(new Vector3(1, 1, 0));
            ds.Write<Vector3>(new Vector3(-1, -1, 0));
            ds.Write<Vector3>(new Vector3(1, -1, 0));
            garagemesh.UnlockVertexBuffer();
            ds = garagemesh.LockIndexBuffer(LockFlags.None);
            ds.Write<Int16>(0); ds.Write<Int16>(1); ds.Write<Int16>(2);
            ds.Write<Int16>(0); ds.Write<Int16>(4); ds.Write<Int16>(3);
            ds.Write<Int16>(0); ds.Write<Int16>(2); ds.Write<Int16>(4);
            ds.Write<Int16>(0); ds.Write<Int16>(3); ds.Write<Int16>(1);
            ds.Write<Int16>(0); ds.Write<Int16>(4); ds.Write<Int16>(2);
            ds.Write<Int16>(0); ds.Write<Int16>(3); ds.Write<Int16>(4);
            garagemesh.UnlockIndexBuffer();

            System.IO.Directory.SetCurrentDirectory(currentFolder);
        }
        
        /// <summary>
        /// This event will be fired immediately after the Direct3D device has been 
        /// reset, which will happen after a lost device scenario. This is the best location to 
        /// create Pool.Default resources since these resources need to be reloaded whenever 
        /// the device is lost. Resources created here should be released in the OnLostDevice 
        /// event. 
        /// </summary>
        private void OnResetDevice(object sender, DeviceEventArgs e)
        {
            // Create a sprite to help batch calls when drawing many lines of text
            textSprite = new Sprite(e.Device);

            // Set the fill mode
            if (isWireframe)
                e.Device.SetRenderState(RenderState.FillMode, FillMode.Wireframe);
            else
                e.Device.SetRenderState(RenderState.FillMode, FillMode.Solid);


            // Setup a light
            Light l = new Light();
            l.Type = LightType.Directional;
            l.Direction = new Vector3(0.0f, -1.0f, 1.0f);
            l.Diffuse = Color.White;
            e.Device.SetLight(0, l);
            e.Device.EnableLight(0, true);

            SetCameraAndViews(e.Device);

        }
        // set (and reset) the camera
        private void SetCameraAndViews(Device d)
        {

            // Setup the camera's projection parameters
            float aspectRatio = (float)panelRenderTarget.Width / (float)panelRenderTarget.Height;
            camera.SetProjectionParameters((float)Math.PI / 4, aspectRatio, objectRadius / 64.0f, objectRadius * 200.0f);
            camera.SetViewParameters(new Vector3(0, 0, -2.2f * objectRadius), Vector3.Zero);
            camera.SetWindow(panelRenderTarget.Width, panelRenderTarget.Height);

            d.SetTransform(TransformState.Projection, camera.ProjectionMatrix);
            d.SetTransform(TransformState.View, camera.ViewMatrix); 
        }

        /// <summary>
        /// This event function will be called fired after the Direct3D device has 
        /// entered a lost state and before Device.Reset() is called. Resources created
        /// in the OnResetDevice callback should be released here, which generally includes all 
        /// Pool.Default resources. See the "Lost Devices" section of the documentation for 
        /// information about lost devices.
        /// </summary>
        private void OnLostDevice(object sender, EventArgs e)
        {
            if (textSprite != null)
            {
                textSprite.Dispose();
                textSprite = null;
            }
        }

        /// <summary>
        /// This callback function will be called immediately after the Direct3D device has 
        /// been destroyed, which generally happens as a result of application termination or 
        /// windowed/full screen toggles. Resources created in the OnCreateDevice callback 
        /// should be released here, which generally includes all Pool.Managed resources. 
        /// </summary>
        private void OnDestroyDevice(object sender, EventArgs e)
        {
            if (!isDestroyed)
            {
                if (rootFrame != null)
                {
                    Frame.DestroyHierarchy(rootFrame, new AnimationAllocation(this));
                    rootFrame = null;
                }
                if (controller != null)
                {
                    controller.Dispose();
                    controller = null;
                }
                isDestroyed = true;
            }

            if (arrowmesh != null)
            {
                arrowmesh.Dispose();
                arrowmesh = null;
            }
            if (spheremesh != null)
            {
                spheremesh.Dispose();
                spheremesh = null;
            }

            if (defaultTexture != null)
            {
                defaultTexture.Dispose();
                defaultTexture = null;
            }
        }

        /// <summary>
        /// This callback function will be called once at the beginning of every frame. This is the
        /// best location for your application to handle updates to the scene, but is not 
        /// intended to contain actual rendering calls, which should instead be placed in the 
        /// OnFrameRender callback.  
        /// </summary>
        public void OnFrameMove(Device device, double appTime, float elapsedTime)
        {
            // Update the camera's position based on user input 
            camera.FrameMove(elapsedTime);
            manipulator.Update();
#if false
            // Build the world matrix
            Matrix worldMatrix = Matrix.Translation(-objectCenter);
            worldMatrix *= camera.WorldMatrix;

            // Set world matrix
            device.SetTransform(TransformState.World, worldMatrix);
#else
            Matrix worldMatrix;
            worldMatrix = Matrix.Translation(-objectCenter);
            worldMatrix *= camera.WorldMatrix;
            device.SetTransform(TransformState.World, worldMatrix);
            //device.SetTransform(TransformState.Projection, camera.ProjectionMatrix);
            device.SetTransform(TransformState.View, camera.ViewMatrix); 
#endif
            if (elapsedTime > 0.0f)
            {
                //if (rootFrame.AnimationController != null)
                //    rootFrame.AnimationController.AdvanceTime(elapsedTime);

                UpdateFrameMatrices(rootFrame, Matrix.Identity, worldMatrix);
            }

        }
        /// <summary>Update the frames matrices and combine it with it's parents</summary>
        private void UpdateFrameMatrices(AnimationFrame frame, Matrix parentMatrix, Matrix worldMatrix)
        {
            if (frame == null) return;

            frame.Update(parentMatrix, worldMatrix);

            if (frame.Sibling != null)
            {
                UpdateFrameMatrices(frame.Sibling as AnimationFrame, parentMatrix, worldMatrix);
            }

            if (frame.FirstChild != null)
            {
                UpdateFrameMatrices(frame.FirstChild as AnimationFrame,
                    frame.InheritedTransformationMatrix, frame.CombinedTransformationMatrix);
            }
        }

        /// <summary>
        /// render a HardPoint
        /// </summary>
        /// <param name="mesh"></param>
        /// <param name="parent"></param>
        private void DrawHardPoint(AnimationFrame frame)
        {
            //if (!frame.HP.Valid) return; // skip invalid hardpoints

            //if (frame.HP.Type == HardPointType.Garage)
            //{
            //    DrawGarage(frame); return;
            //}
            if (frame.HP.Type == HardPointType.Light)
                if (!bShowLights) return;

            Device device = sampleFramework.Device;
            
            FillMode oldfm = (FillMode)device.GetRenderState(RenderState.FillMode);
            //if (oldfm != FillMode.Wireframe)
            //    device.SetRenderState(RenderState.FillMode, FillMode.Wireframe);

            device.SetTransform(TransformState.World, frame.CombinedTransformationMatrix);

            int colorop = device.GetTextureStageState(0, TextureStage.ColorOperation);
            int alphaop = device.GetTextureStageState(0, TextureStage.AlphaOperation);
            Cull cullmode = device.GetRenderState<Cull>(RenderState.CullMode);

            if (frame == SelectedFrame)
            {
                device.SetRenderState(RenderState.ColorVertex, true);
                device.SetRenderState(RenderState.Lighting, false);
                device.SetRenderState(RenderState.CullMode, Cull.None);
            }

            device.SetTexture(0, null);

            device.Material = lightmat;

            ExtendedMaterial[] materials = null;
            bool bHasMat = false;
            Mesh rendermesh = arrowmesh;

            if (frame.HP.Type == HardPointType.Light)
            {
                if (frame != SelectedFrame)
                {
                    device.SetRenderState(RenderState.FillMode, FillMode.Solid);
                    device.SetRenderState(RenderState.AlphaBlendEnable, true);
                    device.SetRenderState(RenderState.SourceBlend, Blend.SourceAlpha);
                    device.SetRenderState(RenderState.DestinationBlend, Blend.InverseSourceAlpha);
                }

                rendermesh = spheremesh;
                HardPointLight hpl = frame.HP as HardPointLight;
                Color4 lcolor = hpl.LightData.color;
                lightmat.Diffuse = new Color4(0.6f, lcolor.Red, lcolor.Green, lcolor.Blue);
                bHasMat = true;
            }

            float size = tbSize.Value;
            size = size / tbSize.Maximum / 2.0f;

            if (frame.HP.Type == HardPointType.Garage)
            {
                rendermesh = planemesh;
                materials = planemesh.GetMaterials();
                size *= 2;

                device.SetRenderState(RenderState.FillMode, FillMode.Solid);
                device.SetRenderState(RenderState.AlphaBlendEnable, true);
                device.SetRenderState(RenderState.SourceBlend, Blend.SourceAlpha);
                device.SetRenderState(RenderState.DestinationBlend, Blend.InverseSourceAlpha);
            }
            if (materials == null)
                materials = arrowmesh.GetMaterials();
            else
                bHasMat = true;
            #region MOVE THIS IN THE UPDATE FRAME and store it in HP.Transformation
            Matrix view = device.GetTransform(TransformState.World);
            // get the scaling center
            Vector3 vscale;
            Quaternion qrotate;
            Vector3 vtrans;
            view.Decompose(out vscale, out qrotate, out vtrans);
            
            vscale *= size * objectRadius;

            // scale it
            Matrix mscale = Matrix.Transformation(vtrans, Quaternion.Identity, vscale, Vector3.Zero, Quaternion.Identity, Vector3.Zero);

            Matrix scaleview = view * mscale;
            device.SetTransform(TransformState.World, scaleview);
            #endregion
            
            for (int i = 0; i < materials.Length; ++i)
            {
                if (frame != SelectedFrame)
                {
                    if (bHasMat && frame.HP.Type == HardPointType.Light)
                        device.Material = lightmat;
                    else
                        device.Material = materials[i].MaterialD3D;
                }
                rendermesh.DrawSubset(i);
            }

            device.SetRenderState(RenderState.FillMode, oldfm);

            if (frame == SelectedFrame)
            {
                device.SetRenderState(RenderState.ColorVertex, true);
                device.SetRenderState(RenderState.Lighting, true);
                device.SetRenderState(RenderState.CullMode, cullmode);
            }

            // reset the world trans
            device.SetTransform(TransformState.World, view);
        }
        private void DrawGarage(AnimationFrame frame)
        {
            Device device = sampleFramework.Device;

            HardPointGarage gar = frame.HP as HardPointGarage;
            if (gar == null) return; // should raise an exception 
            throw new NotImplementedException();
            
        }
        /// <summary>Render a mesh container</summary>
        private void DrawMeshContainer(AnimationMeshContainer mesh, AnimationFrame parent)
        {
            if (parent.Style == RenderStyle.Hide) return;

            Device device = sampleFramework.Device;
            // first check for skinning
            if (mesh.SkinInfo != null)
            {
                if (mesh.NumberInfluences == 1)
                    device.SetRenderState(RenderState.VertexBlend, VertexBlend.Weights0);
                else
                    device.SetRenderState(RenderState.VertexBlend , (VertexBlend)(mesh.NumberInfluences - 1));

                if (mesh.NumberInfluences > 0)
                    device.SetRenderState(RenderState.IndexedVertexBlendEnable, true);

                BoneCombination[] bones = mesh.GetBones();

                for (int iAttrib = 0; iAttrib < mesh.NumberAttributes; iAttrib++)
                {
                    // first, get world matrices
                    for (int iPaletteEntry = 0; iPaletteEntry < mesh.NumberPaletteEntries;
                        ++iPaletteEntry)
                    {
                        int iMatrixIndex = bones[iAttrib].BoneIds[iPaletteEntry];
                        if (iMatrixIndex != -1)
                        {
                            device.SetTransform(iPaletteEntry, 
                                mesh.GetOffsetMatrices()[iMatrixIndex] *
                                mesh.GetFrames()[iMatrixIndex].CombinedTransformationMatrix);

                        }
                    }

                    // Setup the material
                    device.Material = mesh.GetMaterials()[bones[iAttrib].AttributeId].MaterialD3D;
                    device.SetTexture(0, mesh.GetTextures()[bones[iAttrib].AttributeId]);

                    // Finally draw the subset
                    mesh.MeshData.Mesh.DrawSubset(iAttrib);
                }
            }
            else
            {
                // Standard mesh, just draw it using FF
                device.SetRenderState(RenderState.VertexBlend, VertexBlend.Disable);

                Cull oldcull = device.GetRenderState<Cull>(RenderState.CullMode);
                FillMode oldfm = (FillMode)device.GetRenderState(RenderState.FillMode);
                FillMode newfm = oldfm;
                if (!isWireframe)
                    newfm = (parent.Style == RenderStyle.Wireframe) ? FillMode.Wireframe : FillMode.Solid;

                if (oldfm != newfm)
                    device.SetRenderState(RenderState.FillMode, newfm);

                // Set up transforms
                device.SetTransform(TransformState.World, parent.CombinedTransformationMatrix);

                if (parent.IsConvexHull)
                {
                    //device.Material = cvhmat;
                    //device.SetRenderState(RenderState.Ambient, cvhmat.Ambient.ToArgb());
                    ////device.SetRenderState(RenderState.Lighting, false);
                    //device.SetRenderState(RenderState.PointScaleEnable, false);
                    //device.SetRenderState(RenderState.PointSize, 32.0f);
                    //device.SetStreamSource(0,mesh.Mesh.Mesh.VertexBuffer,0,mesh.Mesh.Mesh.BytesPerVertex);
                    //device.DrawPrimitives(PrimitiveType.PointList,0,mesh.Mesh.Mesh.VertexCount);
                    device.SetRenderState<Cull>(RenderState.CullMode, Cull.None);
                }
                ExtendedMaterial[] materials = mesh.GetMaterials();
                for (int i = 0; i < materials.Length; ++i)
                {
                    if (parent.IsConvexHull)
                        device.Material = cvhmat;
                    else
                        device.Material = materials[i].MaterialD3D;
                    device.SetTexture(0, mesh.GetTextures()[i]);
                    mesh.MeshData.Mesh.DrawSubset(i);
                }
                device.SetRenderState<Cull>(RenderState.CullMode, oldcull);

                // restore FillMode
                if (oldfm != newfm)
                    device.SetRenderState(RenderState.FillMode, oldfm);

                // render outlined edges
                if (parent == SelectedFrame)
                {
                    // save states
                    int colorop = device.GetTextureStageState(0, TextureStage.ColorOperation);
                    int alphaop = device.GetTextureStageState(0, TextureStage.AlphaOperation);
                    Cull cullmode = device.GetRenderState<Cull>(RenderState.CullMode);


                    Material mat = new Material();
                    mat.Emissive = new Color4(1.0f, 1.0f, 0.0f, 0.0f);
                    mat.Ambient = new Color4(0.0f, 0.0f, 0.0f, 0.0f);
                    mat.Specular = new Color4(0.0f, 0.0f, 0.0f, 0.0f);
                    mat.Diffuse = new Color4(0.0f, 0.0f, 0.0f, 0.0f);
                    mat.Power = 1.0f;

                    device.Material = mat;
                    device.SetTexture(0, null);

                    device.SetRenderState(RenderState.ColorVertex, true);
                    device.SetRenderState(RenderState.Lighting, false);
                    device.SetRenderState(RenderState.FillMode, FillMode.Wireframe);
                    device.SetRenderState(RenderState.CullMode, Cull.None);

                    //device.SetTextureStageState(0, TextureStage.ColorOperation, TextureOperation.Disable);
                    //device.SetTextureStageState(0, TextureStage.AlphaOperation, TextureOperation.Disable);

                    Matrix mvieworig = device.GetTransform(TransformState.View);
                    Matrix mview = mvieworig;
                    mview.M43 -= parent.boundingSphere.Radius / 40.0f; // todo tune this ?
                    device.SetTransform(TransformState.View, mview);
                    for (int i = 0; i < materials.Length; ++i)
                    {
                        mesh.MeshData.Mesh.DrawSubset(i);
                    }

                    // reset states

                    //device.SetTextureStageState(0, TextureStage.ColorOperation, colorop);
                    //device.SetTextureStageState(0, TextureStage.AlphaOperation, alphaop);
                    device.SetRenderState(RenderState.ColorVertex, true);
                    device.SetRenderState(RenderState.FillMode, oldfm);
                    device.SetRenderState(RenderState.Lighting, true);
                    device.SetRenderState(RenderState.CullMode, cullmode);
                    device.SetTransform(TransformState.View, mvieworig);
                }
            }
        }

        /// <summary>Draw a frame and all child and sibling frames</summary>
        private void DrawFrame(AnimationFrame frame)
        {
            if (frame == null) return;

            if (frame.HP.Type != HardPointType.None)
            {
                DrawHardPoint(frame);
            }
            else
            {
                AnimationMeshContainer mesh = frame.MeshContainer as AnimationMeshContainer;
                while (mesh != null)
                {
                    DrawMeshContainer(mesh, frame);

                    mesh = mesh.NextMeshContainer as AnimationMeshContainer;
                }
            }
            if (frame.Sibling != null)
            {
                DrawFrame(frame.Sibling as AnimationFrame);
            }

            if (frame.FirstChild != null)
            {
                DrawFrame(frame.FirstChild as AnimationFrame);
            }
        }

        /// <summary>
        /// This callback function will be called at the end of every frame to perform all the 
        /// rendering calls for the scene, and it will also be called if the window needs to be 
        /// repainted. After this function has returned, the sample framework will call 
        /// Device.Present to display the contents of the next buffer in the swap chain
        /// </summary>
        public void OnFrameRender(Device device, double appTime, float elapsedTime)
        {
                bool beginSceneCalled = false;

                // Clear the render target and the zbuffer 
                device.Clear(ClearFlags.ZBuffer | ClearFlags.Target, 0x002D32AA, 1.0f, 0);
                try
                {
                    device.BeginScene();
                    beginSceneCalled = true;

                    // Render the animation
                    DrawFrame(rootFrame);

                    // Show frame rate
                    RenderText();

                    // draw the manipulator
                    manipulator.Draw();

                    // Show UI
                    //hud.OnRender(elapsedTime);
                    //sampleUi.OnRender(elapsedTime);
                }
                finally
                {
                    if (beginSceneCalled)
                        device.EndScene();
                }
        }

        /// <summary>
        /// Render the help and statistics text. This function uses the Font object for 
        /// efficient text rendering.
        /// </summary>
        private void RenderText()
        {
            TextHelper txtHelper = new TextHelper(statsFont, textSprite, 15);

            // Output statistics
            txtHelper.Begin();
            txtHelper.SetInsertionPoint(5,5);
            txtHelper.SetForegroundColor(System.Drawing.Color.Turquoise);
            txtHelper.DrawTextLine(sampleFramework.FrameStats);
            txtHelper.DrawTextLine(sampleFramework.DeviceStats);

            // Draw help
            if (isHelpShowing)
            {
                txtHelper.SetInsertionPoint(10, sampleFramework.BackBufferSurfaceDescription.Height-15*6);
                txtHelper.SetForegroundColor(System.Drawing.Color.DarkOrange);
                txtHelper.DrawTextLine("Controls (F1 to hide):");

                txtHelper.SetInsertionPoint(40, sampleFramework.BackBufferSurfaceDescription.Height-15*5);
                txtHelper.DrawTextLine("Rotate mesh: Left click drag");
                txtHelper.DrawTextLine("Rotate camera: right click drag");
                txtHelper.DrawTextLine("Zoom: mouse wheel");
                txtHelper.DrawTextLine("Quit: Esc");
                txtHelper.DrawTextLine("Hide help: F1");
            }
            else
            {
                txtHelper.SetInsertionPoint(10, sampleFramework.BackBufferSurfaceDescription.Height-15*2);
                txtHelper.SetForegroundColor(System.Drawing.Color.White);
                txtHelper.DrawTextLine("Press F1 for help");
            }

            txtHelper.SetInsertionPoint(10, 65);
            txtHelper.SetForegroundColor(System.Drawing.Color.DarkOrange);
            
            if (SelectedFrame != null)
                txtHelper.DrawTextLine("selected frame: {0}", SelectedFrame.Name); 

            //txtHelper.DrawTextLine("Number Faces: {0}\n", (vidMemEnhancedMesh == null) ? 0 : vidMemEnhancedMesh.FaceCount);
            //txtHelper.DrawTextLine("Number Vertices: {0}\n", (vidMemEnhancedMesh == null) ? 0 : vidMemEnhancedMesh.VertexCount);

            txtHelper.End();
        }

        /// <summary>
        /// As a convenience, the sample framework inspects the incoming windows messages for
        /// keystroke messages and decodes the message parameters to pass relevant keyboard
        /// messages to the application.  The framework does not remove the underlying keystroke 
        /// messages, which are still passed to the application's MsgProc callback.
        /// </summary>
        private void OnKeyEvent(object sender, System.Windows.Forms.KeyEventArgs e)
        {
            switch(e.KeyCode)
            {
                case System.Windows.Forms.Keys.F1:
                    isHelpShowing = !isHelpShowing;
                    break;
            }
        }

        /// <summary>
        /// Before handling window messages, the sample framework passes incoming windows 
        /// messages to the application through this callback function. If the application sets 
        /// noFurtherProcessing to true, the sample framework will not process the message
        /// </summary>
        public IntPtr OnMsgProc(IntPtr hWnd, NativeMethods.WindowMessage msg, IntPtr wParam, IntPtr lParam, ref bool noFurtherProcessing)
        {

            //if (msg == NativeMethods.WindowMessage.MouseWheel)
            //    System.Diagnostics.Debug.WriteLine("OnMsgproc got MouseWheel" + lParam.ToString());
            // Pass all remaining windows messages to camera so it can respond to user input
            camera.HandleMessages(hWnd, msg, wParam, lParam);

            return IntPtr.Zero;
        }

        /// <summary>
        /// Will call into the sample framework's window proc
        /// </summary>
        protected override void WndProc(ref Message m)
        {
            //if (m.Msg == (int)NativeMethods.WindowMessage.MouseWheel)
            //    System.Diagnostics.Debug.WriteLine("wndproc got MouseWheel" + m.LParam.ToString());

            //if (m.Msg == (int)NativeMethods.WindowMessage.MouseWheel2)
            //    System.Diagnostics.Debug.WriteLine("wndproc got MouseWheel2");
            sampleFramework.WindowsProcedure(ref m);
            base.WndProc(ref m);
        }

        #region App Idle Events
        /// <summary>Fired when the application first gets idle</summary>
        public void OnApplicationIdle(object sender, EventArgs e)
        {
            while (AppStillIdle)
            {
                // Render a frame during idle time (no messages are waiting)
                sampleFramework.Render3DEnvironment();
            }
        }
        /// <summary>Checks to see if the application is still idle</summary>
        private bool AppStillIdle
        {
            get
            {
                NativeMethods.Message msg;
                return !NativeMethods.PeekMessage(out msg, IntPtr.Zero, 0, 0, 0);
            }
        }
        #endregion
        
        /// <summary>
        /// Initializes the application
        /// </summary>
        public void InitializeApplication()
        {
            // Hook the events
        }

        /// <summary>Fired when the fill mode is changed</summary>
        private void OnFillmodeChanged()
        {
            sampleFramework.Device.SetRenderState(RenderState.FillMode, isWireframe ? FillMode.Wireframe : FillMode.Solid);
        }

        /// <summary>
        /// Generate the skinned mesh information
        /// </summary>
        public void GenerateSkinnedMesh(AnimationMeshContainer mesh)
        {
            if (mesh.SkinInfo == null)
                throw new ArgumentException();  // There is nothing to generate

            //MeshOptimizeFlags oflags = MeshOptimizeFlags.VertexCache;
            MeshFlags flags = 0;

            Capabilities caps = sampleFramework.DeviceCaps;
            if (caps.VertexShaderVersion >= new Version(1, 1))
            {
                flags |= MeshFlags.Managed;
            }
            else
            {
                flags |= MeshFlags.SystemMemory;
            }

            int numMaxFaceInfl;
            using (IndexBuffer ib = mesh.MeshData.Mesh.IndexBuffer)
            {
                numMaxFaceInfl = mesh.SkinInfo.GetMaxFaceInfluences(ib,
                    mesh.MeshData.Mesh.FaceCount);
            }
            // 12 entry palette guarantees that any triangle (4 independent 
            // influences per vertex of a tri) can be handled
            numMaxFaceInfl = (int)Math.Min(numMaxFaceInfl, 12);

            if (caps.MaxVertexBlendMatrixIndex + 1 >= numMaxFaceInfl)
            {
                mesh.NumberPaletteEntries = (int)Math.Min((caps.
                    MaxVertexBlendMatrixIndex + 1) / 2,
                    mesh.SkinInfo.BoneCount);

                flags |= MeshFlags.Managed;
            }

            int influences = 0;
            BoneCombination[] bones = null;

            // Use ConvertToBlendedMesh to generate a drawable mesh
            MeshData data = mesh.MeshData;
            data = new MeshData(mesh.SkinInfo.ConvertToIndexedBlendedMesh(data.Mesh, mesh.NumberPaletteEntries,
                mesh.GetAdjacency(), out influences,  out bones));

            // Store this info
            mesh.NumberInfluences = influences;
            mesh.SetBones(bones);

            // Get the number of attributes
            mesh.NumberAttributes = bones.Length;

            mesh.MeshData = data;
        }

        /// <summary>This method will set the bone matrices for a frame</summary>
        private void SetupBoneMatrices(AnimationFrame frame)
        {
            // First do the mesh container this frame contains (if it does)
            if (frame.MeshContainer != null)
            {
                SetupBoneMatrices(frame.MeshContainer as AnimationMeshContainer);
            }
            // Next do any siblings this frame may contain
            if (frame.Sibling != null)
            {
                SetupBoneMatrices(frame.Sibling as AnimationFrame);
            }
            // Finally do the children of this frame
            if (frame.FirstChild != null)
            {
                SetupBoneMatrices(frame.FirstChild as AnimationFrame);
            }
        }
        private void SetupBoundingBoxes(AnimationFrame frame)
        {
            // do the frame itself
            frame.ComputeBoundingSphere();
            // do any siblings this frame may contain
            if (frame.Sibling != null)
            {
                SetupBoundingBoxes(frame.Sibling as AnimationFrame);
            }
            // do the children of this frame
            if (frame.FirstChild != null)
            {
                SetupBoundingBoxes(frame.FirstChild as AnimationFrame);
            }

        }
        /// <summary>Sets the bone matrices for a mesh container</summary>
        private void SetupBoneMatrices(AnimationMeshContainer mesh)
        {
            // Is there skin information?  If so, setup the matrices
            if (mesh.SkinInfo != null)
            {
                int numberBones = mesh.SkinInfo.BoneCount;

                AnimationFrame[] frameMatrices = new AnimationFrame[numberBones];
                for (int i = 0; i < numberBones; i++)
                {
                    AnimationFrame frame = rootFrame.FindChild(
                        mesh.SkinInfo.GetBoneName(i)) as AnimationFrame;

                    if (frame == null)
                        throw new InvalidOperationException("Could not find valid bone.");

                    frameMatrices[i] = frame;
                }
                mesh.SetFrames(frameMatrices);
            }
        }

        /// <summary>
        /// Entry point to the program. Initializes everything and goes into a message processing 
        /// loop. Idle time is used to render the scene.
        /// </summary>
        [STAThread]
        static int Main() 
        {
            System.Windows.Forms.Application.EnableVisualStyles();
            using(Framework sampleFramework = new Framework())
            {
                AMTForm sample = new AMTForm(sampleFramework);
                // Set the callback functions. These functions allow the sample framework to notify
                // the application about device changes, user input, and windows messages.  The 
                // callbacks are optional so you need only set callbacks for events you're interested 
                // in. However, if you don't handle the device reset/lost callbacks then the sample 
                // framework won't be able to reset your device since the application must first 
                // release all device resources before resetting.  Likewise, if you don't handle the 
                // device created/destroyed callbacks then the sample framework won't be able to 
                // recreate your device resources.
                sampleFramework.Disposing += new EventHandler(sample.OnDestroyDevice);
                sampleFramework.DeviceLost += new EventHandler(sample.OnLostDevice);
                sampleFramework.DeviceCreated += new DeviceEventHandler(sample.OnCreateDevice);
                sampleFramework.DeviceReset += new DeviceEventHandler(sample.OnResetDevice);
                sampleFramework.SetWndProcCallback(new WndProcCallback(sample.OnMsgProc));

                sampleFramework.SetCallbackInterface(sample);
                //try
                //{

                    // Show the cursor and clip it when in full screen
                    sampleFramework.SetCursorSettings(true, true);

                    // Initialize
                    sample.InitializeApplication();
                    sample.Show();
                    // Initialize the sample framework and create the desired window and Direct3D 
                    // device for the application. Calling each of these functions is optional, but they
                    // allow you to set several options which control the behavior of the sampleFramework.
                    sampleFramework.Initialize( true, true, true ); // Parse the command line, handle the default hotkeys, and show msgboxes
                    //sampleFramework.CreateWindow("Allegiance Modeling Tool",sample);
                    
                    sampleFramework.SetWindow(sample, sample.panelRenderTarget, sample.panelRenderTarget, true);
                    // Hook the keyboard event
                    sampleFramework.Window.KeyDown += new System.Windows.Forms.KeyEventHandler(sample.OnKeyEvent);
                    
                    sampleFramework.CreateDevice(0, true, sample.panelRenderTarget.Width, sample.panelRenderTarget.Height, sample);

                    // Hook the application's idle event
                    System.Windows.Forms.Application.Idle += new EventHandler(sample.OnApplicationIdle);
                    System.Windows.Forms.Application.Run(sample);
//                }
//#if(DEBUG)
//                catch (Exception e)
//                {
//                    // In debug mode show this error (maybe - depending on settings)
//                    sampleFramework.DisplayErrorMessage(e);
//#else
//            catch
//            {
//                // In release mode fail silently
//#endif
//                    // Ignore any exceptions here, they would have been handled by other areas
//                    return (sampleFramework.ExitCode == 0) ? 1 : sampleFramework.ExitCode; // Return an error code here
//                }

                // Perform any application-level cleanup here. Direct3D device resources are released within the
                // appropriate callback functions and therefore don't require any cleanup code here.
                return sampleFramework.ExitCode;
            }
        }

        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(AMTForm));
            this.toolStrip1 = new System.Windows.Forms.ToolStrip();
            this.newToolStripButton = new System.Windows.Forms.ToolStripButton();
            this.openToolStripButton = new System.Windows.Forms.ToolStripButton();
            this.saveToolStripButton = new System.Windows.Forms.ToolStripButton();
            this.toolStripSeparator = new System.Windows.Forms.ToolStripSeparator();
            this.cutToolStripButton = new System.Windows.Forms.ToolStripButton();
            this.copyToolStripButton = new System.Windows.Forms.ToolStripButton();
            this.pasteToolStripButton = new System.Windows.Forms.ToolStripButton();
            this.toolStripSeparator1 = new System.Windows.Forms.ToolStripSeparator();
            this.helpToolStripButton = new System.Windows.Forms.ToolStripButton();
            this.toolStripSeparator2 = new System.Windows.Forms.ToolStripSeparator();
            this.tsbWireframe = new System.Windows.Forms.ToolStripButton();
            this.toolStripSeparator4 = new System.Windows.Forms.ToolStripSeparator();
            this.tsbCenterView = new System.Windows.Forms.ToolStripButton();
            this.toolStripSeparator3 = new System.Windows.Forms.ToolStripSeparator();
            this.tsbAdd = new System.Windows.Forms.ToolStripSplitButton();
            this.toolStripMenuItem1 = new System.Windows.Forms.ToolStripMenuItem();
            this.lightToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.effectToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.launchToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.garageToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.mountToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.toolStripMenuItem2 = new System.Windows.Forms.ToolStripMenuItem();
            this.importToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.computeToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.toolStripButton2 = new System.Windows.Forms.ToolStripButton();
            this.tsbShowLights = new System.Windows.Forms.ToolStripButton();
            this.toolStripSeparator5 = new System.Windows.Forms.ToolStripSeparator();
            this.tsbManipTransAxis = new System.Windows.Forms.ToolStripButton();
            this.tsbManipTransPlane = new System.Windows.Forms.ToolStripButton();
            this.tsbManipRotate = new System.Windows.Forms.ToolStripButton();
            this.tsbManipScaleAxis = new System.Windows.Forms.ToolStripButton();
            this.tsbManipScalePlane = new System.Windows.Forms.ToolStripButton();
            this.tsbManipScaleUni = new System.Windows.Forms.ToolStripButton();
            this.toolStripSeparator6 = new System.Windows.Forms.ToolStripSeparator();
            this.tvFrames = new System.Windows.Forms.TreeView();
            this.iconList = new System.Windows.Forms.ImageList(this.components);
            this.cmsNode = new System.Windows.Forms.ContextMenuStrip(this.components);
            this.listView1 = new System.Windows.Forms.ListView();
            this.splitContainer3 = new System.Windows.Forms.SplitContainer();
            this.splitContainer1 = new System.Windows.Forms.SplitContainer();
            this.panelRenderTarget = new System.Windows.Forms.Panel();
            this.ofdMain = new System.Windows.Forms.OpenFileDialog();
            this.ofdCVH = new System.Windows.Forms.OpenFileDialog();
            this.toolStrip1.SuspendLayout();
            this.splitContainer3.Panel1.SuspendLayout();
            this.splitContainer3.Panel2.SuspendLayout();
            this.splitContainer3.SuspendLayout();
            this.splitContainer1.Panel1.SuspendLayout();
            this.splitContainer1.Panel2.SuspendLayout();
            this.splitContainer1.SuspendLayout();
            this.SuspendLayout();
            // 
            // toolStrip1
            // 
            this.toolStrip1.GripStyle = System.Windows.Forms.ToolStripGripStyle.Hidden;
            this.toolStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.newToolStripButton,
            this.openToolStripButton,
            this.saveToolStripButton,
            this.toolStripSeparator,
            this.cutToolStripButton,
            this.copyToolStripButton,
            this.pasteToolStripButton,
            this.toolStripSeparator1,
            this.helpToolStripButton,
            this.toolStripSeparator2,
            this.tsbWireframe,
            this.toolStripSeparator4,
            this.tsbCenterView,
            this.toolStripSeparator3,
            this.tsbAdd,
            this.toolStripButton2,
            this.tsbShowLights,
            this.toolStripSeparator5,
            this.tsbManipTransAxis,
            this.tsbManipTransPlane,
            this.tsbManipRotate,
            this.tsbManipScaleAxis,
            this.tsbManipScalePlane,
            this.tsbManipScaleUni,
            this.toolStripSeparator6});
            this.toolStrip1.Location = new System.Drawing.Point(0, 0);
            this.toolStrip1.Name = "toolStrip1";
            this.toolStrip1.Size = new System.Drawing.Size(856, 25);
            this.toolStrip1.TabIndex = 3;
            this.toolStrip1.Text = "toolStrip1";
            // 
            // newToolStripButton
            // 
            this.newToolStripButton.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.newToolStripButton.Image = ((System.Drawing.Image)(resources.GetObject("newToolStripButton.Image")));
            this.newToolStripButton.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.newToolStripButton.Name = "newToolStripButton";
            this.newToolStripButton.Size = new System.Drawing.Size(23, 22);
            this.newToolStripButton.Text = "&New";
            this.newToolStripButton.Click += new System.EventHandler(this.newToolStripButton_Click);
            // 
            // openToolStripButton
            // 
            this.openToolStripButton.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.openToolStripButton.Image = ((System.Drawing.Image)(resources.GetObject("openToolStripButton.Image")));
            this.openToolStripButton.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.openToolStripButton.Name = "openToolStripButton";
            this.openToolStripButton.Size = new System.Drawing.Size(23, 22);
            this.openToolStripButton.Text = "&Open";
            this.openToolStripButton.Click += new System.EventHandler(this.openToolStripButton_Click);
            // 
            // saveToolStripButton
            // 
            this.saveToolStripButton.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.saveToolStripButton.Image = ((System.Drawing.Image)(resources.GetObject("saveToolStripButton.Image")));
            this.saveToolStripButton.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.saveToolStripButton.Name = "saveToolStripButton";
            this.saveToolStripButton.Size = new System.Drawing.Size(23, 22);
            this.saveToolStripButton.Text = "&Save";
            // 
            // toolStripSeparator
            // 
            this.toolStripSeparator.Name = "toolStripSeparator";
            this.toolStripSeparator.Size = new System.Drawing.Size(6, 25);
            // 
            // cutToolStripButton
            // 
            this.cutToolStripButton.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.cutToolStripButton.Image = ((System.Drawing.Image)(resources.GetObject("cutToolStripButton.Image")));
            this.cutToolStripButton.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.cutToolStripButton.Name = "cutToolStripButton";
            this.cutToolStripButton.Size = new System.Drawing.Size(23, 22);
            this.cutToolStripButton.Text = "C&ut";
            // 
            // copyToolStripButton
            // 
            this.copyToolStripButton.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.copyToolStripButton.Image = ((System.Drawing.Image)(resources.GetObject("copyToolStripButton.Image")));
            this.copyToolStripButton.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.copyToolStripButton.Name = "copyToolStripButton";
            this.copyToolStripButton.Size = new System.Drawing.Size(23, 22);
            this.copyToolStripButton.Text = "&Copy";
            // 
            // pasteToolStripButton
            // 
            this.pasteToolStripButton.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.pasteToolStripButton.Image = ((System.Drawing.Image)(resources.GetObject("pasteToolStripButton.Image")));
            this.pasteToolStripButton.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.pasteToolStripButton.Name = "pasteToolStripButton";
            this.pasteToolStripButton.Size = new System.Drawing.Size(23, 22);
            this.pasteToolStripButton.Text = "&Paste";
            // 
            // toolStripSeparator1
            // 
            this.toolStripSeparator1.Name = "toolStripSeparator1";
            this.toolStripSeparator1.Size = new System.Drawing.Size(6, 25);
            // 
            // helpToolStripButton
            // 
            this.helpToolStripButton.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.helpToolStripButton.Image = ((System.Drawing.Image)(resources.GetObject("helpToolStripButton.Image")));
            this.helpToolStripButton.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.helpToolStripButton.Name = "helpToolStripButton";
            this.helpToolStripButton.Size = new System.Drawing.Size(23, 22);
            this.helpToolStripButton.Text = "He&lp";
            this.helpToolStripButton.Click += new System.EventHandler(this.helpToolStripButton_Click);
            // 
            // toolStripSeparator2
            // 
            this.toolStripSeparator2.Name = "toolStripSeparator2";
            this.toolStripSeparator2.Size = new System.Drawing.Size(6, 25);
            // 
            // tsbWireframe
            // 
            this.tsbWireframe.Alignment = System.Windows.Forms.ToolStripItemAlignment.Right;
            this.tsbWireframe.CheckOnClick = true;
            this.tsbWireframe.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.tsbWireframe.Image = ((System.Drawing.Image)(resources.GetObject("tsbWireframe.Image")));
            this.tsbWireframe.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.tsbWireframe.Name = "tsbWireframe";
            this.tsbWireframe.Size = new System.Drawing.Size(23, 22);
            this.tsbWireframe.Text = "Toggle wireframe";
            this.tsbWireframe.ToolTipText = "Toogle wireframe";
            this.tsbWireframe.CheckedChanged += new System.EventHandler(this.tsbWireframe_CheckedChanged);
            // 
            // toolStripSeparator4
            // 
            this.toolStripSeparator4.Name = "toolStripSeparator4";
            this.toolStripSeparator4.Size = new System.Drawing.Size(6, 25);
            // 
            // tsbCenterView
            // 
            this.tsbCenterView.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.tsbCenterView.Image = ((System.Drawing.Image)(resources.GetObject("tsbCenterView.Image")));
            this.tsbCenterView.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.tsbCenterView.Name = "tsbCenterView";
            this.tsbCenterView.Size = new System.Drawing.Size(23, 22);
            this.tsbCenterView.Text = "center view";
            this.tsbCenterView.Click += new System.EventHandler(this.tsbCenterView_Click);
            // 
            // toolStripSeparator3
            // 
            this.toolStripSeparator3.Name = "toolStripSeparator3";
            this.toolStripSeparator3.Size = new System.Drawing.Size(6, 25);
            // 
            // tsbAdd
            // 
            this.tsbAdd.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Text;
            this.tsbAdd.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.toolStripMenuItem1,
            this.toolStripMenuItem2});
            this.tsbAdd.Image = ((System.Drawing.Image)(resources.GetObject("tsbAdd.Image")));
            this.tsbAdd.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.tsbAdd.Name = "tsbAdd";
            this.tsbAdd.Size = new System.Drawing.Size(54, 22);
            this.tsbAdd.Text = "Add...";
            // 
            // toolStripMenuItem1
            // 
            this.toolStripMenuItem1.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.lightToolStripMenuItem,
            this.effectToolStripMenuItem,
            this.launchToolStripMenuItem,
            this.garageToolStripMenuItem,
            this.mountToolStripMenuItem});
            this.toolStripMenuItem1.Name = "toolStripMenuItem1";
            this.toolStripMenuItem1.Size = new System.Drawing.Size(131, 22);
            this.toolStripMenuItem1.Text = "Hard point";
            // 
            // lightToolStripMenuItem
            // 
            this.lightToolStripMenuItem.Name = "lightToolStripMenuItem";
            this.lightToolStripMenuItem.Size = new System.Drawing.Size(113, 22);
            this.lightToolStripMenuItem.Text = "Light";
            // 
            // effectToolStripMenuItem
            // 
            this.effectToolStripMenuItem.Name = "effectToolStripMenuItem";
            this.effectToolStripMenuItem.Size = new System.Drawing.Size(113, 22);
            this.effectToolStripMenuItem.Text = "Effect";
            // 
            // launchToolStripMenuItem
            // 
            this.launchToolStripMenuItem.Name = "launchToolStripMenuItem";
            this.launchToolStripMenuItem.Size = new System.Drawing.Size(113, 22);
            this.launchToolStripMenuItem.Text = "Launch";
            // 
            // garageToolStripMenuItem
            // 
            this.garageToolStripMenuItem.Name = "garageToolStripMenuItem";
            this.garageToolStripMenuItem.Size = new System.Drawing.Size(113, 22);
            this.garageToolStripMenuItem.Text = "Garage";
            // 
            // mountToolStripMenuItem
            // 
            this.mountToolStripMenuItem.Name = "mountToolStripMenuItem";
            this.mountToolStripMenuItem.Size = new System.Drawing.Size(113, 22);
            this.mountToolStripMenuItem.Text = "Mount";
            // 
            // toolStripMenuItem2
            // 
            this.toolStripMenuItem2.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.importToolStripMenuItem,
            this.computeToolStripMenuItem});
            this.toolStripMenuItem2.Name = "toolStripMenuItem2";
            this.toolStripMenuItem2.Size = new System.Drawing.Size(131, 22);
            this.toolStripMenuItem2.Text = "Hull";
            // 
            // importToolStripMenuItem
            // 
            this.importToolStripMenuItem.Name = "importToolStripMenuItem";
            this.importToolStripMenuItem.Size = new System.Drawing.Size(203, 22);
            this.importToolStripMenuItem.Text = "Import";
            // 
            // computeToolStripMenuItem
            // 
            this.computeToolStripMenuItem.Name = "computeToolStripMenuItem";
            this.computeToolStripMenuItem.Size = new System.Drawing.Size(203, 22);
            this.computeToolStripMenuItem.Text = "Compute from selection";
            // 
            // toolStripButton2
            // 
            this.toolStripButton2.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Text;
            this.toolStripButton2.Image = ((System.Drawing.Image)(resources.GetObject("toolStripButton2.Image")));
            this.toolStripButton2.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.toolStripButton2.Name = "toolStripButton2";
            this.toolStripButton2.Size = new System.Drawing.Size(64, 22);
            this.toolStripButton2.Text = "Load CVH";
            this.toolStripButton2.Click += new System.EventHandler(this.toolStripButton2_Click);
            // 
            // tsbShowLights
            // 
            this.tsbShowLights.Alignment = System.Windows.Forms.ToolStripItemAlignment.Right;
            this.tsbShowLights.Checked = true;
            this.tsbShowLights.CheckOnClick = true;
            this.tsbShowLights.CheckState = System.Windows.Forms.CheckState.Checked;
            this.tsbShowLights.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.tsbShowLights.Image = ((System.Drawing.Image)(resources.GetObject("tsbShowLights.Image")));
            this.tsbShowLights.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.tsbShowLights.Name = "tsbShowLights";
            this.tsbShowLights.Size = new System.Drawing.Size(23, 22);
            this.tsbShowLights.Text = "Toogle lights";
            this.tsbShowLights.CheckedChanged += new System.EventHandler(this.tsbShowLights_CheckedChanged);
            // 
            // toolStripSeparator5
            // 
            this.toolStripSeparator5.Name = "toolStripSeparator5";
            this.toolStripSeparator5.Size = new System.Drawing.Size(6, 25);
            // 
            // tsbManipTransAxis
            // 
            this.tsbManipTransAxis.CheckOnClick = true;
            this.tsbManipTransAxis.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.tsbManipTransAxis.Image = ((System.Drawing.Image)(resources.GetObject("tsbManipTransAxis.Image")));
            this.tsbManipTransAxis.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.tsbManipTransAxis.Name = "tsbManipTransAxis";
            this.tsbManipTransAxis.Size = new System.Drawing.Size(23, 22);
            this.tsbManipTransAxis.CheckedChanged += new System.EventHandler(this.tsbManip_CheckedChanged);
            // 
            // tsbManipTransPlane
            // 
            this.tsbManipTransPlane.CheckOnClick = true;
            this.tsbManipTransPlane.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.tsbManipTransPlane.Image = ((System.Drawing.Image)(resources.GetObject("tsbManipTransPlane.Image")));
            this.tsbManipTransPlane.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.tsbManipTransPlane.Name = "tsbManipTransPlane";
            this.tsbManipTransPlane.Size = new System.Drawing.Size(23, 22);
            this.tsbManipTransPlane.CheckedChanged += new System.EventHandler(this.tsbManip_CheckedChanged);
            // 
            // tsbManipRotate
            // 
            this.tsbManipRotate.CheckOnClick = true;
            this.tsbManipRotate.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.tsbManipRotate.Image = ((System.Drawing.Image)(resources.GetObject("tsbManipRotate.Image")));
            this.tsbManipRotate.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.tsbManipRotate.Name = "tsbManipRotate";
            this.tsbManipRotate.Size = new System.Drawing.Size(23, 22);
            this.tsbManipRotate.CheckedChanged += new System.EventHandler(this.tsbManip_CheckedChanged);
            // 
            // tsbManipScaleAxis
            // 
            this.tsbManipScaleAxis.CheckOnClick = true;
            this.tsbManipScaleAxis.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.tsbManipScaleAxis.Image = ((System.Drawing.Image)(resources.GetObject("tsbManipScaleAxis.Image")));
            this.tsbManipScaleAxis.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.tsbManipScaleAxis.Name = "tsbManipScaleAxis";
            this.tsbManipScaleAxis.Size = new System.Drawing.Size(23, 22);
            this.tsbManipScaleAxis.CheckedChanged += new System.EventHandler(this.tsbManip_CheckedChanged);
            // 
            // tsbManipScalePlane
            // 
            this.tsbManipScalePlane.CheckOnClick = true;
            this.tsbManipScalePlane.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.tsbManipScalePlane.Image = ((System.Drawing.Image)(resources.GetObject("tsbManipScalePlane.Image")));
            this.tsbManipScalePlane.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.tsbManipScalePlane.Name = "tsbManipScalePlane";
            this.tsbManipScalePlane.Size = new System.Drawing.Size(23, 22);
            this.tsbManipScalePlane.CheckedChanged += new System.EventHandler(this.tsbManip_CheckedChanged);
            // 
            // tsbManipScaleUni
            // 
            this.tsbManipScaleUni.CheckOnClick = true;
            this.tsbManipScaleUni.DisplayStyle = System.Windows.Forms.ToolStripItemDisplayStyle.Image;
            this.tsbManipScaleUni.Image = ((System.Drawing.Image)(resources.GetObject("tsbManipScaleUni.Image")));
            this.tsbManipScaleUni.ImageTransparentColor = System.Drawing.Color.Magenta;
            this.tsbManipScaleUni.Name = "tsbManipScaleUni";
            this.tsbManipScaleUni.Size = new System.Drawing.Size(23, 22);
            this.tsbManipScaleUni.CheckedChanged += new System.EventHandler(this.tsbManip_CheckedChanged);
            // 
            // toolStripSeparator6
            // 
            this.toolStripSeparator6.Name = "toolStripSeparator6";
            this.toolStripSeparator6.Size = new System.Drawing.Size(6, 25);
            // 
            // tvFrames
            // 
            this.tvFrames.Dock = System.Windows.Forms.DockStyle.Fill;
            this.tvFrames.ImageIndex = 0;
            this.tvFrames.ImageList = this.iconList;
            this.tvFrames.Location = new System.Drawing.Point(0, 0);
            this.tvFrames.Name = "tvFrames";
            this.tvFrames.SelectedImageIndex = 0;
            this.tvFrames.Size = new System.Drawing.Size(199, 364);
            this.tvFrames.TabIndex = 5;
            this.tvFrames.MouseUp += new System.Windows.Forms.MouseEventHandler(this.tvFrames_MouseUp);
            this.tvFrames.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.tvFrames_AfterSelect);
            // 
            // iconList
            // 
            this.iconList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("iconList.ImageStream")));
            this.iconList.TransparentColor = System.Drawing.Color.Transparent;
            this.iconList.Images.SetKeyName(0, "garage");
            this.iconList.Images.SetKeyName(1, "launch");
            this.iconList.Images.SetKeyName(2, "position");
            this.iconList.Images.SetKeyName(3, "frame");
            this.iconList.Images.SetKeyName(4, "mesh");
            this.iconList.Images.SetKeyName(5, "effect");
            this.iconList.Images.SetKeyName(6, "light");
            // 
            // cmsNode
            // 
            this.cmsNode.Name = "cmsNode";
            this.cmsNode.Size = new System.Drawing.Size(61, 4);
            this.cmsNode.ItemClicked += new System.Windows.Forms.ToolStripItemClickedEventHandler(this.cmsNode_ItemClicked);
            // 
            // listView1
            // 
            this.listView1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.listView1.LargeImageList = this.iconList;
            this.listView1.Location = new System.Drawing.Point(0, 0);
            this.listView1.Name = "listView1";
            this.listView1.Size = new System.Drawing.Size(199, 208);
            this.listView1.SmallImageList = this.iconList;
            this.listView1.TabIndex = 7;
            this.listView1.UseCompatibleStateImageBehavior = false;
            this.listView1.View = System.Windows.Forms.View.Tile;
            // 
            // splitContainer3
            // 
            this.splitContainer3.Dock = System.Windows.Forms.DockStyle.Fill;
            this.splitContainer3.Location = new System.Drawing.Point(0, 0);
            this.splitContainer3.Name = "splitContainer3";
            this.splitContainer3.Orientation = System.Windows.Forms.Orientation.Horizontal;
            // 
            // splitContainer3.Panel1
            // 
            this.splitContainer3.Panel1.Controls.Add(this.tvFrames);
            // 
            // splitContainer3.Panel2
            // 
            this.splitContainer3.Panel2.Controls.Add(this.listView1);
            this.splitContainer3.Size = new System.Drawing.Size(199, 576);
            this.splitContainer3.SplitterDistance = 364;
            this.splitContainer3.TabIndex = 9;
            // 
            // splitContainer1
            // 
            this.splitContainer1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.splitContainer1.Location = new System.Drawing.Point(0, 25);
            this.splitContainer1.Name = "splitContainer1";
            // 
            // splitContainer1.Panel1
            // 
            this.splitContainer1.Panel1.Controls.Add(this.splitContainer3);
            // 
            // splitContainer1.Panel2
            // 
            this.splitContainer1.Panel2.Controls.Add(this.panelRenderTarget);
            this.splitContainer1.Size = new System.Drawing.Size(856, 576);
            this.splitContainer1.SplitterDistance = 199;
            this.splitContainer1.TabIndex = 12;
            // 
            // panelRenderTarget
            // 
            this.panelRenderTarget.AutoSizeMode = System.Windows.Forms.AutoSizeMode.GrowAndShrink;
            this.panelRenderTarget.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.panelRenderTarget.Dock = System.Windows.Forms.DockStyle.Fill;
            this.panelRenderTarget.Location = new System.Drawing.Point(0, 0);
            this.panelRenderTarget.Name = "panelRenderTarget";
            this.panelRenderTarget.Size = new System.Drawing.Size(653, 576);
            this.panelRenderTarget.TabIndex = 0;
            this.panelRenderTarget.MouseWheel += new System.Windows.Forms.MouseEventHandler(this.panelRenderTarget_MouseWheel);
            this.panelRenderTarget.MouseMove += new System.Windows.Forms.MouseEventHandler(this.panelRenderTarget_MouseMove);
            this.panelRenderTarget.MouseDown += new System.Windows.Forms.MouseEventHandler(this.panelRenderTarget_MouseDown);
            this.panelRenderTarget.MouseUp += new System.Windows.Forms.MouseEventHandler(this.panelRenderTarget_MouseUp);
            // 
            // ofdCVH
            // 
            this.ofdCVH.Filter = "CVH Files|*.cvh";
            // 
            // AMTForm
            // 
            this.ClientSize = new System.Drawing.Size(856, 601);
            this.Controls.Add(this.splitContainer1);
            this.Controls.Add(this.toolStrip1);
            this.Name = "AMTForm";
            this.toolStrip1.ResumeLayout(false);
            this.toolStrip1.PerformLayout();
            this.splitContainer3.Panel1.ResumeLayout(false);
            this.splitContainer3.Panel2.ResumeLayout(false);
            this.splitContainer3.ResumeLayout(false);
            this.splitContainer1.Panel1.ResumeLayout(false);
            this.splitContainer1.Panel2.ResumeLayout(false);
            this.splitContainer1.ResumeLayout(false);
            this.ResumeLayout(false);
            this.PerformLayout();

        }

        protected void LoadTV(AnimationFrame frame)
        {
            LoadTV(frame, false);
        }
        protected void LoadTV(AnimationFrame frame, TreeNodeCollection parent)
        {
            TreeNode tn = new TreeNode(frame.Name);
            tn.Tag = frame;
            tn.ImageKey = "frame";


            if (frame.MeshContainer != null)
                tn.ImageKey = "mesh";

            switch (frame.HP.Type)
            {
                case HardPointType.Position: tn.ImageKey = "position"; break;
                case HardPointType.Garage:   tn.ImageKey = "garage"; break;
                case HardPointType.Launch:   tn.ImageKey = "launch"; break;
                case HardPointType.Light:    tn.ImageKey = "light"; break;
                case HardPointType.Effect:   tn.ImageKey = "effect"; break;
                default: break;
            }
            if (frame.HP.Type != HardPointType.None)
            {
                if (frame.HP.Valid)
                {
                    ListViewItem li = listView1.Items.Add(frame.HP.Name, tn.ImageKey);
                    li.Tag = frame;
                }
            }

            tn.SelectedImageKey = tn.ImageKey;

            if (frame.FirstChild != null)
            {
                LoadTV((AnimationFrame)frame.FirstChild, tn.Nodes);
            }

            //if (frame.HP.Valid)
                parent.Add(tn);

            if (frame.Sibling != null)
                LoadTV((AnimationFrame)frame.Sibling, parent);
        }
        protected void LoadTV(AnimationFrame frame, bool bclear)
        {
            if (bclear)
            {
                SelectedFrame = null;
                tvFrames.Nodes.Clear();
                listView1.Clear();
            }
            LoadTV(frame,tvFrames.Nodes);
        }

        private void cmsNode_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            AnimationFrame frame = (AnimationFrame)tvFrames.SelectedNode.Tag;
            frame.Style = (RenderStyle)e.ClickedItem.Tag;
            //System.Diagnostics.Debug.WriteLine("menu selected for " + frame.Name +" value = "+frame.Style.ToString());
        }

        private void tvFrames_MouseUp(object sender, MouseEventArgs e)
        {
            // Show menu only if the right mouse button is clicked.
            if (e.Button == MouseButtons.Right)
            {
                if (tvFrames.SelectedNode == null) return;

                // Point where the mouse is clicked.
                Point p = new Point(e.X, e.Y);

                // Get the node that the user has clicked.
                TreeNode node = tvFrames.GetNodeAt(p);
                if (node == tvFrames.SelectedNode)
                {
                    cmsNode.Show(tvFrames, p);
                }
            }
        }

        private void tvFrames_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (tvFrames.SelectedNode != null)
                SelectedFrame = (AnimationFrame)tvFrames.SelectedNode.Tag;
            else
                SelectedFrame = null;
        }

        private void panelRenderTarget_MouseDown(object sender, MouseEventArgs e)
        {
            panelRenderTarget.Focus();
            manipulator.HandleMouseMessage(e,true);
            camera.HandleMouseMessages(e, true, false);
        }

        private void panelRenderTarget_MouseUp(object sender, MouseEventArgs e)
        {
            manipulator.HandleMouseMessage(e,false);
            camera.HandleMouseMessages(e, true, true);
        }

        private void panelRenderTarget_MouseMove(object sender, MouseEventArgs e)
        {
            manipulator.HandleMouseMessage(e,null);
            camera.HandleMouseMessages(e, false, false);
        }

        void panelRenderTarget_MouseWheel(object sender, MouseEventArgs e)
        {
            camera.HandleMouseMessages(e, false, true);
        }

        private void helpToolStripButton_Click(object sender, EventArgs e)
        {
            isHelpShowing = !isHelpShowing;
        }

        private void openToolStripButton_Click(object sender, EventArgs e)
        {
            ofdMain.Filter = "X Files|*.x|MDL Files|*.mdl";
            if (ofdMain.ShowDialog() != DialogResult.OK) return;

            NewScene();
            Device device = sampleFramework.Device;
            // Load the mesh

            //string path = Utility.FindMediaFile("acs05\\acs05.x");
            //string path = Utility.FindMediaFile("fig13\\fig13.x");
            //string path = Utility.FindMediaFile(@"C:\dev\alleg\AllegianceSource1p2\src\Artwork\256\acs05.x");
            //string path = Utility.FindMediaFile("tiny.x");
            string path = ofdMain.FileName;

            // Change the current directory to the mesh's directory so we can
            // find the textures.
            string currentFolder = System.IO.Directory.GetCurrentDirectory();
            System.IO.FileInfo info = new System.IO.FileInfo(path);
            System.IO.Directory.SetCurrentDirectory(info.Directory.FullName);
            this.Text = "AMT II - a0.85 - " + info.Name;
            bool usefrmprefix = false;
            // MDL File
            if (path.EndsWith(".mdl"))
            {
                MDLFile mdlfile = new MDLFile();
                bool success = false;
                try
                {
                    success = mdlfile.ReadFromFile(path);
                }
                catch (Exception ee)
                {
                    MessageBox.Show("error loading " + path + " err = " + ee.Message);
                    return;
                }

                if (mdlfile.RootObject.type == MDLType.mdl_image)
                    success = false;
                if (mdlfile.RootObject.type == MDLType.mdl_empty)
                    success = false;

                if (!success)
                {
                    MessageBox.Show("unsupported MDL file to load as 3D object: " + path);
                    return;
                }
                // MDL to internal format
                rootFrame = AMTHelper.MDLtoFrame(new AnimationAllocation(this), mdlfile, device);
            }
            // X File
            else if (path.EndsWith(".x"))
            {
                usefrmprefix = true;
                AnimationAllocation alloc = new AnimationAllocation(this);
                try
                {
                    rootFrame = (AnimationFrame)Frame.LoadHierarchyFromX(device, path, MeshFlags.SystemMemory, alloc, null, out controller);
                }
                catch (Exception ee)
                {
                    MessageBox.Show("error loading " + path + ". Error = " + ee.Message);
                    return;
                }
                if (controller != null)
                    AMTHelper.MoveAnimToKey(controller, 1);
            }

            AMTHelper.MergeGarageFrames(rootFrame, rootFrame, usefrmprefix);

            LoadTV(rootFrame, true);

            // Calculate the center and radius of a bounding sphere
            //BoundingSphere s = Frame.CalculateBoundingSphere(rootFrame);
            SetupBoundingBoxes(rootFrame);
            objectCenter = rootFrame.boundingSphere.Center;
            objectRadius = rootFrame.boundingSphere.Radius;

            // Setup the matrices for animation
            SetupBoneMatrices(rootFrame);
            isDestroyed = false;

            // Restore the original folder
            System.IO.Directory.SetCurrentDirectory(currentFolder);

            SetCameraAndViews(device);
        }
        /// <summary>
        /// clear everything
        /// </summary>
        private void NewScene()
        {
            if (rootFrame != null)
            {
                Frame.DestroyHierarchy(rootFrame, new AnimationAllocation(this));
                rootFrame = null;
            }
            if (controller != null)
            {
                controller.Dispose();
                controller = null;
            }
            SelectedFrame = null;
            tvFrames.Nodes.Clear();
            listView1.Clear();
        }

        private void tsbCenterView_Click(object sender, EventArgs e)
        {
            if (rootFrame == null) return;

            if (SelectedFrame != null)
            {
                Vector3 vscale;
                Quaternion qrot;
                Vector3 vtrans;
                SelectedFrame.InheritedTransformationMatrix.Decompose(out vscale, out qrot, out vtrans);
                objectCenter = vtrans;
            }
            else
                objectCenter = rootFrame.boundingSphere.Center;

            SetCameraAndViews(sampleFramework.Device);
        }

        private void toolStripButton2_Click(object sender, EventArgs e)
        {
            string currentFolder = System.IO.Directory.GetCurrentDirectory();

            ofdCVH.Filter = "CVH Files|*.cvh";
            if (ofdCVH.ShowDialog() != DialogResult.OK) return;

            try
            {
                cvh = CVH.FromFile(ofdCVH.FileName);
            }
            catch
            {
                MessageBox.Show("error reading " + ofdCVH.FileName);
            }
            IEAllocateHierarchy ialloc =  new AnimationAllocation(this);
            AnimationFrame cvhframe = AMTHelper.CVHtoFrame(ialloc, cvh, sampleFramework.Device);
            AMTHelper.MergeGarageFrames(cvhframe, cvhframe, false);
            if (rootFrame != null)
            {
                AnimationFrame newroot = ialloc.CreateFrame("/") as AnimationFrame;
                newroot.AppendChild(rootFrame);
                rootFrame = newroot;
                rootFrame.AppendChild(cvhframe);
            }
            else
                rootFrame = cvhframe;

            LoadTV(rootFrame, true);

            // Calculate the center and radius of a bounding sphere
            //BoundingSphere s = Frame.CalculateBoundingSphere(rootFrame);
            SetupBoundingBoxes(rootFrame);
            objectCenter = rootFrame.boundingSphere.Center;
            objectRadius = rootFrame.boundingSphere.Radius;

            // Setup the matrices for animation
            SetupBoneMatrices(rootFrame);
            isDestroyed = false;

            // Restore the original folder
            System.IO.Directory.SetCurrentDirectory(currentFolder);

            SetCameraAndViews(sampleFramework.Device);
        }

        private void newToolStripButton_Click(object sender, EventArgs e)
        {
            NewScene();
        }

        private void tsbShowLights_CheckedChanged(object sender, EventArgs e)
        {
            bShowLights = tsbShowLights.Checked;
        }

        private void tsbWireframe_CheckedChanged(object sender, EventArgs e)
        {
            isWireframe = tsbWireframe.Checked;
            OnFillmodeChanged();
        }

        private void tsbManip_CheckedChanged(object sender, EventArgs e)
        {
            ToolStripButton tsb = (ToolStripButton)sender;
            TransformationMode mode = (TransformationMode)tsb.Tag;
            if (tsb.Checked)
                manipulator.EnabledModes |= mode;
            else
                manipulator.EnabledModes &= ~mode;
        }
    }
}
