//
//      coded by un
//            --------------
//                     mindshifter.com
//


using System;
using System.Collections.Generic;
using SlimDX;
using SlimDX.Direct3D9;
using Microsoft.Samples.DirectX.UtilityToolkit;

namespace Mindshifter
{
    internal class PickEffect
    {
        private Framework mFramework;
        private Effect                  mEffect;

        internal PickEffect(Framework framework)
        {
            mFramework = framework;

            mFramework.DeviceCreated += new DeviceEventHandler(CreateEffect);
            if ((mFramework.Device != null) && !mFramework.Device.Disposed)
                CreateEffect(this, null);
        }

        private void CreateEffect(object sender, EventArgs e)
        {
            //CompiledEffect ce = Effect.FromString(EffectSource, new CompilerMacro[0],
            //    null, CompilerOptions.None, TargetPlatform.Windows);

            //mEffect = new Effect(mFramework.GraphicsDevice, ce.GetEffectCode(), CompilerOptions.None, null);
            mEffect = Effect.FromString(mFramework.Device, EffectSource, ShaderFlags.None);
        }

        public void Render(Queue<PickRenderable> renderables)
        {
            mEffect.Begin(FX.None);//      SaveStateMode.SaveState);
            mEffect.BeginPass(0); // CurrentTechnique.Passes[0].Begin();

            foreach (PickRenderable rend in renderables)
            {
                mEffect.SetValue<Matrix>("WVP", rend.WVP);// Parameters["WVP"].SetValue(rend.WVP);
                Vector4 v4 = rend.PickColor.ToVector4();
                mEffect.SetValue<Vector4>("PickID", v4);//Parameters["PickID"].SetValue(rend.PickColor.ToVector4());
                mEffect.CommitChanges();

                rend.Render();
            }

            mEffect.EndPass(); // CurrentTechnique.Passes[0].End();
            mEffect.End();
        }

        private static string EffectSource =
    @"  float4x4 WVP;
        float4 PickID;

        float4 VS(float4 Position : POSITION0) : POSITION0
        {
            return mul(Position, WVP);
        }

        float4 PS(float4 Position : POSITION0) : COLOR0
        {
            return PickID;
        }

        Technique Pick
        {
            Pass Pick
            {
                AlphaBlendEnable = False;
                FillMode = Solid;
                ZEnable = True;
                ZWriteEnable = True;
                ZFunc = LessEqual;
                MultiSampleAntialias = False;

                VertexShader = compile vs_1_1 VS();
                PixelShader = compile ps_2_0 PS();
            }
        }";
    }
}