
#ifndef _VERTEXGENERATOR_H_
#define _VERTEXGENERATOR_H_

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
class CVertexGenerator
{
////////////////////////////////////////////////////////////////////////////////////////////////////
private:
	static CVertexGenerator mSingleInstance;		// CVertexGenerator singleton.

////////////////////////////////////////////////////////////////////////////////////////////////////
public:
	////////////////////////////////////////////////////////////////////////////////////////////
	enum EPredefinedDynamicBufferType
	{
		// UI dynamic buffers.
		ePDBT_UITexVB = 0,
		ePDBT_UIFillVB,
		ePDBT_UIFontVB,
		
		// 3D dynamic buffers.
		ePDBT_D3DVB,
		ePDBT_D3DLitVB,

		ePDBT_NumPredefinedDynamicBuffers
	};

////////////////////////////////////////////////////////////////////////////////////////////////////
private:
	struct SPredefinedDynamicBufferConfig
	{
		DWORD dwBufferType;
		DWORD dwNumElements;
		DWORD dwElementSize;
		DWORD dwFormat;
	};

	////////////////////////////////////////////////////////////////////////////////////////////
	struct SVertexGeneratorState
	{
		bool bInitialised;

		CVBIBManager::SVBIBHandle hPredefinedDynBuffers[ePDBT_NumPredefinedDynamicBuffers];
//		CVBIBManager::SVBIBHandle hUITexVertsVB;
//		CVBIBManager::SVBIBHandle hUIFillVertsVB;
//		CVBIBManager::SVBIBHandle hUIFontVertsVB;
//		CVBIBManager::SVBIBHandle hD3DVB;
//		CVBIBManager::SVBIBHandle hD3DLitVB;

	};

	////////////////////////////////////////////////////////////////////////////////////////////
	static SPredefinedDynamicBufferConfig m_sDynBufferConfig[ ePDBT_NumPredefinedDynamicBuffers ];
	SVertexGeneratorState m_sVGState;

////////////////////////////////////////////////////////////////////////////////////////////////////
public:
	////////////////////////////////////////////////////////////////////////////////////////////
	CVertexGenerator();
	virtual ~CVertexGenerator();

	////////////////////////////////////////////////////////////////////////////////////////////
	static inline CVertexGenerator * Get() { return &mSingleInstance;	}

	////////////////////////////////////////////////////////////////////////////////////////////
	void Initialise( );

	////////////////////////////////////////////////////////////////////////////////////////////
	void GenerateUITexturedVertices(	const TEXHANDLE hTexture,
										const WinRect & rectToDraw,
										const bool bSetStream );

	////////////////////////////////////////////////////////////////////////////////////////////
	void GenerateFillVertices(	const WinRect & rectToFill, 
								const bool bSetStream,
								Color fillColour );

	////////////////////////////////////////////////////////////////////////////////////////////
	void GenerateFillVerticesD3DColor(	const WinRect & rectToFill,
										const bool bSetStream,
										DWORD d3dColor );

	////////////////////////////////////////////////////////////////////////////////////////////
	inline CVBIBManager::SVBIBHandle * GetPredefinedDynamicBuffer( EPredefinedDynamicBufferType eBufferType )	
	{ 
		return &m_sVGState.hPredefinedDynBuffers[eBufferType]; 
	}
};

#endif // _VERTEXGENERATOR_H_