﻿using System;
using System.Drawing;
using System.IO;
using SlimDX;
using SlimDX.Direct3D9;
using Font = SlimDX.Direct3D9.Font;

using Microsoft.Samples.DirectX.UtilityToolkit;
using System.Text.RegularExpressions;

using Allegiance;
namespace MDLEditor
{
    public class MDLResourceCache : ResourceCache
    {
        #region Creation
        private MDLResourceCache() { } // Don't allow creation
        private static MDLResourceCache localObject = null;
        public static new MDLResourceCache GetGlobalInstance()
        {
            if (localObject == null)
                localObject = new MDLResourceCache();

            return localObject;
        }
        #endregion
        /// <summary>Create a texture from a file - renaming extension</summary>
        public Texture CreateTextureFromFile(Device device, string filename, bool rename)
        {
            string matfilename = filename;
            if (matfilename.EndsWith(".ppm") & rename) // special case
            {
                matfilename = matfilename.Replace(".ppm", ".bmp");
            }
            return CreateTextureFromFile(device, matfilename);
        }
        public override Texture CreateTextureFromFile(Device device, string filename)
        {
            if (filename.EndsWith(".mdl"))
            {
                return CreateTextureFromMDLFile(device, filename);
            }
            else
                return CreateTextureFromFileEx(device, filename, D3DX.Default, D3DX.Default, D3DX.Default, Usage.None,
                    Format.Unknown, Pool.Managed, (Filter)D3DX.Default, (Filter)D3DX.Default, 0);
        }
        public Texture CreateTextureFromMDLFile(Device device, string filename)
        {
            // Search the cache first
            foreach (CachedTexture ct in textureCache.Keys)
            {
                if ((string.Compare(ct.Source, filename, true) == 0) &&
                    ct.Type == ResourceType.Texture)
                {
                    // A match was found, return that
                    return textureCache[ct] as Texture;
                }
            }

            MDLFile mdlfile = new MDLFile();
            if (mdlfile.ReadFromFile(filename))
            {
                if (mdlfile.RootObject.type != MDLType.mdl_image)
                    throw new ArgumentException("not a BMP MDL file");
                MemoryStream ms = mdlfile.RootObject.image.ToBMPMemoryStream();
                int ck = 0;// TODO

                ImageInformation info = new ImageInformation();
                Texture t = null;
                try
                {
                    t = Texture.FromStream(device, ms, (int)ms.Length, D3DX.Default, D3DX.Default, D3DX.Default, Usage.None, Format.A8R8G8B8, Pool.Managed, (Filter)D3DX.Default, (Filter)D3DX.Default, ck, out info);
                }
                catch
                {
                    System.Diagnostics.Debug.WriteLine("from stream failed: " + info.ImageFileFormat.ToString());
                }
                CachedTexture entry = new CachedTexture();
                entry.Source = filename;
                entry.Type = ResourceType.Texture;

                textureCache.Add(entry, t);

                return t;
            }
            else
                throw new ArgumentException(filename);
        }
    }
}
